//==================================================================
//
//	File name	:	imagebox.js
//
//	Summary		:	ImageBox class
//
//	Ceated		:	2018/11/27
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================
"use strict";

(function (imagebox) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	imagebox.STYLE_ORIGINAL = 0;
	imagebox.STYLE_AUTO = 1;
	imagebox.STYLE_FULL = 2;
	
	//==========================================================
	//
	//	Class		:	ImageBox
	//
	//	Summary		:	Class of imagebox
	//
	//==========================================================
	imagebox.ImageBox = class extends jaswin.Control
	{
		//==================================================
		// constructor
		//==================================================
		constructor(sID,			// ID[String]
					oRect,			// * Size and position[FixedRect]
					iWidthStyle,	// * Style of width[int]
					iHeightStyle,	// * Style of height[int]
					sSrc,			// Image source[String]
					sTooltip,		// Tool tip text[String]
					bDraggable,		// Draggable or not[boolean]
					fpDragstart)	// Function on start dragging[function]
		{
			// Base class
			super(true, sID, false, oRect, true, -1);
			
			let sTmpID;
			if(sID)
				sTmpID = sID;
			else
				sTmpID = "";
			
			if(typeof iWidthStyle != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sTmpID),
											"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
			if((iWidthStyle != imagebox.STYLE_ORIGINAL)
			&& (iWidthStyle != imagebox.STYLE_AUTO)
			&& (iWidthStyle != imagebox.STYLE_FULL))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, sTmpID),
											"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
			
			if(typeof iHeightStyle != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sTmpID),
											"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
			if((iHeightStyle != imagebox.STYLE_ORIGINAL)
			&& (iHeightStyle != imagebox.STYLE_AUTO)
			&& (iHeightStyle != imagebox.STYLE_FULL))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, sTmpID),
											"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
			
			if(sSrc)
			{
				if(typeof sSrc != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_IMAGE_SOURCE, sTmpID),
												"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
			}
			
			if(sTooltip)
			{
				if(typeof sTooltip != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, sTmpID),
												"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
			}
			
			if(bDraggable)
			{
				if(typeof bDraggable != "boolean")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_DRAGGABLE, sTmpID),
												"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
				if(fpDragstart)
				{
					if(typeof fpDragstart != "function")
						throw new jaswin.JaswinError(jaswin.ERR_ALERT,
													jaswin.formatMessage(jaswin.ALERT_TYPE_ON_DRAGSTART, sTmpID),
													"imagebox.js", "jaswin.imagebox.ImageBox.constructor");
				}
			}
			
			// Set properties
			this.widthStyle = iWidthStyle;
			this.heightStyle = iHeightStyle;
			this.draggable = bDraggable;			// Draggable or not[boolean]
			this.onDragstart = fpDragstart;			// Function on dragstart[function(Argument:Label class, event)]
			
			// Create element
			this.element = document.createElement("img");
			this.element.style.position = "absolute";
			
			if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.posX + "px";
			else
				this.element.style.left = this.rect.posX + "px";
			
			if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.posY + "px";
			else
				this.element.style.top = this.rect.posY + "px";
			
			this.element.style.width = this.rect.width + "px";
			this.element.style.height = this.rect.height + "px";
			
			this.element.addEventListener("load", this.loadEvent = (e) => {
				this.resize();
			}, false);
			
			if(sSrc)
				this.element.src = sSrc;
			
			if(sTooltip)
				this.element.title = sTooltip;
			
			// Event binding
			if(this.draggable)
			{
				this.element.draggable = true;
				this.element.addEventListener("dragstart", this.dragStartEvent = (e) => {
					this.onDragstart(this, e);
				}, false);
			}
			else
				this.element.draggable = false;
		}
		
		//==================================================
		//	Enable this imagebox
		//==================================================
		enable(bParent)
		{
			super.enable(bParent);
			
			if((this.enabled) && (this.draggable))
			{
				this.element.draggable = true;
				if(!this.dragStartEvent)
				{
					this.element.addEventListener("dragstart", this.dragStartEvent = (e) => {
						this.onDragstart(this, e);
					}, false);
				}
			}
		}
		
		//==================================================
		//	Disable this imagebox
		//==================================================
		disable(bParent)
		{
			super.disable(bParent);
			
			if(this.draggable)
			{
				this.element.draggable = false;
				if(this.dragStartEvent)
				{
					this.element.removeEventListener("dragstart", this.dragStartEvent, false);
					this.dragStartEvent = null;
				}
			}
		}
		
		//==================================================
		//	Get source
		//==================================================
		getSource()
		{
			return this.element.src;
		}
		
		//==================================================
		//	Set source
		//==================================================
		setSource(sSource)
		{
			if(typeof sSource != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_IMAGE_SOURCE),
											"imagebox.js", "jaswin.imagebox.ImageBox.setSource");
			
			this.element.src = sSource;
		}
		
		//==================================================
		//	Resize image
		//==================================================
		resize()
		{
			let iWidth = this.element.naturalWidth;
			let iHeight = this.element.naturalHeight;
			
			if(this.widthStyle == imagebox.STYLE_ORIGINAL)
			{
				// Original image width
				this.element.style.width = iWidth + "px";
				if(this.heightStyle == imagebox.STYLE_ORIGINAL)
					this.element.style.height = iHeight + "px";
				else if(this.heightStyle == imagebox.STYLE_FULL)
					this.element.style.height = this.rect.height + "px";
				else
					this.element.style.height = iHeight + "px";
			}
			else if(this.widthStyle == imagebox.STYLE_FULL)
			{
				// Full width of this control
				this.element.style.width = this.rect.width + "px";
				if(this.heightStyle == imagebox.STYLE_ORIGINAL)
					this.element.style.height = iHeight + "px";
				else if(this.heightStyle == imagebox.STYLE_FULL)
					this.element.style.height = this.rect.height + "px";
				else
					this.element.style.height = Math.round(this.rect.width * (iHeight / iWidth)) + "px";
			}
			else
			{
				// Auto width
				if(this.heightStyle == imagebox.STYLE_ORIGINAL)
				{
					this.element.style.width = iWidth + "px";
					this.element.style.height = iHeight + "px";
				}
				else if(this.heightStyle == imagebox.STYLE_FULL)
				{
					this.element.style.height = this.rect.height + "px";
					this.element.style.width = Math.round(this.rect.height * (iWidth / iHeight)) + "px";
				}
				else
				{
					this.element.style.width = iWidth + "px";
					this.element.style.height = iWidth + "px";
				}
			}
		}
		
		//==================================================
		//	Set tooltip
		//==================================================
		setTooltip(sTooltip)
		{
			if(typeof sTooltip != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOOLTIP, this.id),
											"imagebox.js", "jaswin.imagebox.ImageBox.setTooltip");
			this.element.title = sTooltip;
		}
	}
	
}(jaswin.imagebox = jaswin.imagebox || {}));
